@@ -77,6 +77,8 @@ pub enum Element {
     UnsignedInt(u64),
     /// A response code
     RespCode(RespCode),
+    /// A floating point value
+    Float(f64),
 }
 
 impl Element {
@@ -117,10 +119,10 @@ pub enum ParseError {
 }
 
 #[derive(Debug, PartialEq)]
-/// # Types of Response
+/// # Response types
 ///
-/// A simple response carries the data for one action while a complex response carries data for
-/// multiple actions
+/// A simple response carries the response for a simple query while a pipelined response carries the response
+/// for pipelined queries
 pub enum RawResponse {
     /// A simple query will just hold one element
     SimpleQuery(Element),
@@ -370,6 +372,24 @@ impl<'a> Parser<'a> {
             Err(ParseError::UnexpectedByte)
         }
     }
+    fn parse_next_float(&mut self) -> ParseResult<f64> {
+        let our_float_chunk = self.__get_next_element()?;
+        match String::from_utf8_lossy(our_float_chunk).parse() {
+            Ok(f) => {
+                if self.will_cursor_give_linefeed()? {
+                    self.incr_cursor();
+                    Ok(f)
+                } else {
+                    println!("LF error");
+                    Err(ParseError::UnexpectedByte)
+                }
+            }
+            Err(e) => {
+                println!("Error: {}", e);
+                Err(ParseError::UnexpectedByte)
+            }
+        }
+    }
     /// The cursor should be **at the tsymbol**
     fn parse_next_element(&mut self) -> ParseResult<Element> {
         if let Some(tsymbol) = self.buffer.get(self.cursor) {
@@ -382,6 +402,27 @@ impl<'a> Parser<'a> {
                 b':' => Element::UnsignedInt(self.parse_next_u64()?),
                 b'&' => Element::Array(Array::Recursive(self.parse_next_array()?)),
                 b'!' => Element::RespCode(self.parse_next_respcode()?),
+                b'%' => Element::Float(self.parse_next_float()?),
+                b'^' => {
+                    // hmm, a typed non-null array; let's check the tsymbol
+                    if let Some(array_type) = self.buffer.get(self.cursor) {
+                        // got tsymbol, let's skip it
+                        self.incr_cursor();
+                        match array_type {
+                            b'+' => {
+                                Element::Array(Array::NonNullStr(self.parse_next_nonnull_str()?))
+                            }
+                            b'?' => {
+                                Element::Array(Array::NonNullBin(self.parse_next_nonnull_bin()?))
+                            }
+                            _ => return Err(ParseError::UnknownDatatype),
+                        }
+                    } else {
+                        // if we couldn't fetch a tsymbol, there wasn't enough
+                        // data left
+                        return Err(ParseError::NotEnough);
+                    }
+                }
                 b'@' => {
                     // hmmm, a typed array; let's check the tsymbol
                     if let Some(array_type) = self.buffer.get(self.cursor) {
@@ -415,6 +456,24 @@ impl<'a> Parser<'a> {
             Ok(Some(Self::parse_into_usize(inp)?))
         }
     }
+
+    /// The cursor should have passed the `@+` chars
+    fn parse_next_nonnull_str(&mut self) -> ParseResult<Vec<String>> {
+        let (start, stop) = self.read_line();
+        if let Some(our_size_chunk) = self.buffer.get(start..stop) {
+            // so we have a size chunk; let's get the size
+            let array_size = Self::parse_into_usize(our_size_chunk)?;
+            let mut array = Vec::with_capacity(array_size);
+            for _ in 0..array_size {
+                // no tsymbol, just elements and their sizes
+                array.push(self.parse_next_string()?);
+            }
+            Ok(array)
+        } else {
+            Err(ParseError::NotEnough)
+        }
+    }
+
     /// The cursor should have passed the `@+` chars
     fn parse_next_typed_array_str(&mut self) -> ParseResult<Vec<Option<String>>> {
         let (start, stop) = self.read_line();
@@ -446,6 +505,24 @@ impl<'a> Parser<'a> {
             Err(ParseError::NotEnough)
         }
     }
+
+    /// The cursor should have passed the `@+` chars
+    fn parse_next_nonnull_bin(&mut self) -> ParseResult<Vec<Vec<u8>>> {
+        let (start, stop) = self.read_line();
+        if let Some(our_size_chunk) = self.buffer.get(start..stop) {
+            // so we have a size chunk; let's get the size
+            let array_size = Self::parse_into_usize(our_size_chunk)?;
+            let mut array = Vec::with_capacity(array_size);
+            for _ in 0..array_size {
+                // no tsymbol, just elements and their sizes
+                array.push(self.parse_next_binstr()?);
+            }
+            Ok(array)
+        } else {
+            Err(ParseError::NotEnough)
+        }
+    }
+
     /// The cursor should have passed the tsymbol
     fn parse_next_flat_array(&mut self) -> ParseResult<Vec<FlatElement>> {
         let (start, stop) = self.read_line();
@@ -594,3 +671,39 @@ fn test_typed_str_array_null() {
         ])))
     );
 }
+
+#[test]
+fn test_parse_float() {
+    let packet = b"*1\n%3\n1.1\n";
+    let (parsed, forward) = Parser::new(packet).parse().unwrap();
+    assert_eq!(forward, packet.len());
+    assert_eq!(parsed, RawResponse::SimpleQuery(Element::Float(1.1)))
+}
+
+#[test]
+fn test_parse_nonnull_str() {
+    let packet = b"*1\n^+2\n2\nhi\n5\nthere\n";
+    let (parsed, forward) = Parser::new(packet).parse().unwrap();
+    assert_eq!(forward, packet.len());
+    assert_eq!(
+        parsed,
+        RawResponse::SimpleQuery(Element::Array(Array::NonNullStr(vec![
+            "hi".to_owned(),
+            "there".to_owned()
+        ])))
+    )
+}
+
+#[test]
+fn test_parse_nonnull_bin() {
+    let packet = b"*1\n^?2\n2\nhi\n5\nthere\n";
+    let (parsed, forward) = Parser::new(packet).parse().unwrap();
+    assert_eq!(forward, packet.len());
+    assert_eq!(
+        parsed,
+        RawResponse::SimpleQuery(Element::Array(Array::NonNullBin(vec![
+            "hi".as_bytes().to_owned(),
+            "there".as_bytes().to_owned()
+        ])))
+    )
+}
