@@ -38,7 +38,7 @@
 //!
 //! First add this to your `Cargo.toml` file:
 //! ```toml
-//! skytable = "0.6.1"
+//! skytable = "0.6.2"
 //! ```
 //! Now open up your `src/main.rs` file and establish a connection to the server while also adding some
 //! imports:
@@ -125,7 +125,7 @@
 //!
 //! If you need to use an `async` API, just change your import to:
 //! ```toml
-//! skytable = { version = "0.6.1", features=["async"], default-features=false }
+//! skytable = { version = "0.6.2", features=["async"], default-features=false }
 //! ```
 //! You can now establish a connection by using `skytable::AsyncConnection::new()`, adding `.await`s wherever
 //! necessary. Do note that you'll the [Tokio runtime](https://tokio.rs).
@@ -135,7 +135,7 @@
 //! With this client driver, it is possible to use both sync and `async` APIs **at the same time**. To do
 //! this, simply change your import to:
 //! ```toml
-//! skytable = { version="0.6.1", features=["sync", "async"] }
+//! skytable = { version="0.6.2", features=["sync", "async"] }
 //! ```
 //!
 //! ## TLS
@@ -144,13 +144,13 @@
 //!
 //! ### Using TLS with sync interfaces
 //! ```toml
-//! skytable = { version="0.6.1", features=["sync","ssl"] }
+//! skytable = { version="0.6.2", features=["sync","ssl"] }
 //! ```
 //! You can now use the async [TlsConnection](`sync::TlsConnection`) object.
 //!
 //! ### Using TLS with async interfaces
 //! ```toml
-//! skytable = { version="0.6.1", features=["async","aio-ssl"], default-features=false }
+//! skytable = { version="0.6.2", features=["async","aio-ssl"], default-features=false }
 //! ```
 //! You can now use the async [TlsConnection](`aio::TlsConnection`) object.
 //!
@@ -232,12 +232,12 @@ pub type SkyResult = SkyRawResult<Element>;
 /// ```
 ///
 /// ## Example (async)
-/// ```no_test
+/// ```no_run
 /// use skytable::ConnectionBuilder;
-/// async fn main() {
+/// async fn run() {
 ///     let con =
 ///         ConnectionBuilder::new()
-///         .set_host("127.0.0.1")
+///         .set_host("127.0.0.1".to_string())
 ///         .set_port(2003)
 ///         .set_entity("mykeyspace:mytable".to_string())
 ///         .get_async_connection()
@@ -336,6 +336,74 @@ impl ConnectionBuilder {
     }
 }
 
+cfg_sync! {
+    trait WriteQuerySync {
+        fn write_sync(&self, b: &mut impl std::io::Write) -> IoResult<()>;
+    }
+
+    impl WriteQuerySync for Query {
+        fn write_sync(&self, stream: &mut impl std::io::Write) -> IoResult<()> {
+            // Write the metaframe
+            stream.write_all(b"*1\n")?;
+            // Add the dataframe
+            let number_of_items_in_datagroup = self.len().to_string().into_bytes();
+            stream.write_all(&[b'~'])?;
+            stream.write_all(&number_of_items_in_datagroup)?;
+            stream.write_all(&[b'\n'])?;
+            stream.write_all(self.get_holding_buffer())?;
+            stream.flush()?;
+            Ok(())
+        }
+    }
+
+    impl WriteQuerySync for Pipeline {
+        fn write_sync(&self, stream: &mut impl std::io::Write) -> IoResult<()> {
+            let len = self.len.to_string().into_bytes();
+            stream.write_all(b"*")?;
+            stream.write_all(&len)?;
+            stream.write_all(b"\n")?;
+            stream.write_all(&self.chain)
+        }
+    }
+}
+
+cfg_async! {
+    use core::pin::Pin;
+    use core::future::Future;
+    use tokio::io::AsyncWrite;
+    type FutureRet<'s> = Pin<Box<dyn Future<Output = IoResult<()>> + Send + Sync + 's>>;
+    trait WriteQueryAsync<T: AsyncWrite + Unpin + Send + Sync>: Unpin + Sync + Send {
+        fn write_async<'s>(&'s self, b: &'s mut T) -> FutureRet<'s>;
+    }
+    impl<T: AsyncWrite + Unpin + Send + Sync> WriteQueryAsync<T> for Query {
+        fn write_async<'s>(&'s self, stream: &'s mut T) -> FutureRet {
+            Box::pin(async move {
+                // Write the metaframe
+                stream.write_all(b"*1\n").await?;
+                // Add the dataframe
+                let number_of_items_in_datagroup = self.len().to_string().into_bytes();
+                stream.write_all(&[b'~']).await?;
+                stream.write_all(&number_of_items_in_datagroup).await?;
+                stream.write_all(&[b'\n']).await?;
+                stream.write_all(self.get_holding_buffer()).await?;
+                stream.flush().await?;
+                Ok(())
+            })
+        }
+    }
+    impl<T: AsyncWrite + Unpin + Send + Sync> WriteQueryAsync<T> for Pipeline {
+        fn write_async<'s>(&'s self, stream: &'s mut T) -> FutureRet {
+            Box::pin(async move {
+                let len = self.len.to_string().into_bytes();
+                stream.write_all(b"*").await?;
+                stream.write_all(&len).await?;
+                stream.write_all(b"\n").await?;
+                stream.write_all(&self.chain).await
+            })
+        }
+    }
+}
+
 #[macro_export]
 /// A macro that can be used to easily create queries with _almost_ variadic properties.
 /// Where you'd normally create queries like this:
@@ -472,42 +540,14 @@ impl Query {
     fn get_holding_buffer(&self) -> &[u8] {
         &self.data
     }
-    cfg_async!(
-        /// Write a query to a given stream
-        async fn write_query_to<T>(&self, stream: &mut T) -> IoResult<()>
-        where
-            T: tokio::io::AsyncWrite + Unpin,
-        {
-            // Write the metaframe
-            stream.write_all(b"*1\n").await?;
-            // Add the dataframe
-            let number_of_items_in_datagroup = self.len().to_string().into_bytes();
-            stream.write_all(&[b'~']).await?;
-            stream.write_all(&number_of_items_in_datagroup).await?;
-            stream.write_all(&[b'\n']).await?;
-            stream.write_all(self.get_holding_buffer()).await?;
-            stream.flush().await?;
-            Ok(())
-        }
-    );
-    cfg_sync!(
-        /// Write a query to a given stream
-        fn write_query_to_sync<T>(&self, stream: &mut T) -> IoResult<()>
-        where
-            T: std::io::Write,
-        {
-            // Write the metaframe
-            stream.write_all(b"*1\n")?;
-            // Add the dataframe
-            let number_of_items_in_datagroup = self.len().to_string().into_bytes();
-            stream.write_all(&[b'~'])?;
-            stream.write_all(&number_of_items_in_datagroup)?;
-            stream.write_all(&[b'\n'])?;
-            stream.write_all(self.get_holding_buffer())?;
-            stream.flush()?;
-            Ok(())
-        }
-    );
+    fn write_query_to_writable(&self, buffer: &mut Vec<u8>) {
+        // Add the dataframe element
+        let number_of_items_in_datagroup = self.len().to_string().into_bytes();
+        buffer.extend([b'~']);
+        buffer.extend(&number_of_items_in_datagroup);
+        buffer.extend([b'\n']);
+        buffer.extend(self.get_holding_buffer());
+    }
     cfg_dbg!(
         /// Get the raw bytes of a query
         ///
@@ -564,3 +604,110 @@ cfg_dbg!(
         assert_eq!(ma_query_len, q_len);
     }
 );
+
+/// # Pipeline
+///
+/// A pipeline is a way of queing up multiple queries, sending them to the server at once instead of sending them individually, avoiding
+/// round-trip-times while also simplifying usage in several places. Responses are returned in the order they are sent.
+///
+/// ## Example with the sync API
+///
+/// ```no_run
+/// use skytable::{query, Pipeline, Element, RespCode};
+/// use skytable::sync::Connection;
+///
+/// let mut con = Connection::new("127.0.0.1", 2003).unwrap();
+/// let pipe = Pipeline::new()
+///     .add(query!("set", "x", "100"))
+///     .add(query!("heya", "echo me!"));
+///
+/// let ret = con.run_pipeline(pipe).unwrap();
+/// assert_eq!(
+///     ret,
+///     vec![
+///         Element::RespCode(RespCode::Okay),
+///         Element::String("echo me!".to_owned())
+///     ]
+/// );
+/// ```
+///
+/// ## Example with the async API
+///
+/// ```no_run
+/// use skytable::{query, Pipeline, Element, RespCode};
+/// use skytable::aio::Connection;
+///
+/// async fn run() {
+///     let mut con = Connection::new("127.0.0.1", 2003).await.unwrap();
+///     let pipe = Pipeline::new()
+///         .add(query!("set", "x", "100"))
+///         .add(query!("heya", "echo me!"));
+///
+///     let ret = con.run_pipeline(pipe).await.unwrap();
+///     assert_eq!(
+///         ret,
+///         vec![
+///             Element::RespCode(RespCode::Okay),
+///             Element::String("echo me!".to_owned())
+///         ]
+///     );
+/// }
+/// ```
+///
+pub struct Pipeline {
+    len: usize,
+    chain: Vec<u8>,
+}
+
+impl Pipeline {
+    /// Initializes a new empty pipeline
+    pub fn new() -> Self {
+        Self {
+            len: 0usize,
+            chain: Vec::new(),
+        }
+    }
+    /// Append a query (builder pattern)
+    pub fn add(mut self, query: Query) -> Self {
+        self.len += 1;
+        query.write_query_to_writable(&mut self.chain);
+        self
+    }
+    /// Append a query by taking reference
+    pub fn push(&mut self, query: Query) {
+        self.len += 1;
+        query.write_query_to_writable(&mut self.chain);
+    }
+    /// Returns the number of queries in the pipeline
+    pub fn len(&self) -> usize {
+        self.len
+    }
+    cfg_dbg! {
+        /// Returns the query packet representation of this pipeline
+        ///
+        /// ## Panics
+        ///
+        /// This function will panic if the query is empty
+        pub fn into_raw_query(self) -> Vec<u8> {
+            if self.len == 0 {
+                panic!("The pipeline is empty")
+            } else {
+                let mut v = Vec::with_capacity(self.chain.len() + 4);
+                v.push(b'*');
+                v.extend(self.len.to_string().as_bytes());
+                v.push(b'\n');
+                v.extend(self.chain);
+                v
+            }
+        }
+    }
+}
+
+cfg_dbg! {
+#[test]
+    fn test_pipeline_dbg() {
+        let bytes = b"*2\n~1\n5\nhello\n~1\n5\nworld\n";
+        let pipe = Pipeline::new().add(query!("hello")).add(query!("world"));
+        assert_eq!(pipe.into_raw_query(), bytes);
+    }
+}
