@@ -270,7 +270,7 @@ impl<T: IntoSkyhashBytes, const N: usize> GetIterator<T> for &'static [T; N] {
     }
 }
 
-impl<T: IntoSkyhashBytes> GetIterator<T> for &[T] {
+impl<'a, T: IntoSkyhashBytes> GetIterator<T> for &'a [T] {
     fn get_iter(&self) -> std::slice::Iter<'_, T> {
         self.iter()
     }
@@ -282,20 +282,18 @@ impl<T: IntoSkyhashBytes> GetIterator<T> for Vec<T> {
     }
 }
 
-impl<T: IntoSkyhashBytes> GetIterator<T> for &Vec<T> {
-    fn get_iter(&self) -> std::slice::Iter<'_, T> {
-        self.iter()
-    }
-}
-
 /// Array types
 #[derive(Debug, PartialEq)]
 #[non_exhaustive]
 pub enum Array {
-    //// A binary array (typed array tsymbol `?`, `@` base tsymbol)
+    /// A binary array with nullable elements(typed array tsymbol `?`, `@` base tsymbol)
     Bin(Vec<Option<Vec<u8>>>),
-    /// An unicode string array (typed array tsymbol `+`, `@` base tsymbol)
+    /// A non-null binary array
+    NonNullBin(Vec<Vec<u8>>),
+    /// An unicode string array with nullable elements (typed array tsymbol `+`, `@` base tsymbol)
     Str(Vec<Option<String>>),
+    /// A non-null string array
+    NonNullStr(Vec<String>),
     /// A non-recursive 'flat' array (tsymbol `_`)
     Flat(Vec<FlatElement>),
     /// A recursive array (tsymbol `&`)
@@ -442,10 +440,8 @@ impl<'a> IntoSkyhashBytes for &'a RawString {
 /// use skytable::sync::Connection;
 /// use skytable::actions::Actions;
 ///
-/// fn main() {
-///     let mut con = Connection::new("127.0.0.1", 2003).unwrap();
-///     let mycsv: MyCSV = con.get("mycsv").unwrap();
-/// }
+/// let mut con = Connection::new("127.0.0.1", 2003).unwrap();
+/// let mycsv: MyCSV = con.get("mycsv").unwrap();
 /// ```
 ///
 /// Now, you can use this as you like to turn [`Element`]s into your own (or primitive) types or
